# Generated by roxygen2: do not edit by hand

S3method(curly_brace_na,Spatial)
S3method(curly_brace_na,data.frame)
S3method(curly_brace_na,sf)
S3method(drop_null_geometries,character)
S3method(drop_null_geometries,json)
S3method(ms_clip,SpatialLines)
S3method(ms_clip,SpatialPoints)
S3method(ms_clip,SpatialPolygons)
S3method(ms_clip,character)
S3method(ms_clip,json)
S3method(ms_clip,sf)
S3method(ms_clip,sfc)
S3method(ms_dissolve,SpatialPoints)
S3method(ms_dissolve,SpatialPolygons)
S3method(ms_dissolve,character)
S3method(ms_dissolve,json)
S3method(ms_dissolve,sf)
S3method(ms_dissolve,sfc)
S3method(ms_erase,SpatialLines)
S3method(ms_erase,SpatialPoints)
S3method(ms_erase,SpatialPolygons)
S3method(ms_erase,character)
S3method(ms_erase,json)
S3method(ms_erase,sf)
S3method(ms_erase,sfc)
S3method(ms_explode,SpatialLines)
S3method(ms_explode,SpatialPolygons)
S3method(ms_explode,character)
S3method(ms_explode,json)
S3method(ms_explode,sf)
S3method(ms_explode,sfc)
S3method(ms_filter_fields,SpatialLinesDataFrame)
S3method(ms_filter_fields,SpatialPointsDataFrame)
S3method(ms_filter_fields,SpatialPolygonsDataFrame)
S3method(ms_filter_fields,character)
S3method(ms_filter_fields,json)
S3method(ms_filter_fields,sf)
S3method(ms_filter_islands,SpatialPolygons)
S3method(ms_filter_islands,character)
S3method(ms_filter_islands,json)
S3method(ms_filter_islands,sf)
S3method(ms_filter_islands,sfc)
S3method(ms_innerlines,SpatialPolygons)
S3method(ms_innerlines,character)
S3method(ms_innerlines,json)
S3method(ms_innerlines,sf)
S3method(ms_innerlines,sfc)
S3method(ms_lines,SpatialPolygons)
S3method(ms_lines,character)
S3method(ms_lines,json)
S3method(ms_lines,sf)
S3method(ms_lines,sfc)
S3method(ms_points,SpatialPolygons)
S3method(ms_points,character)
S3method(ms_points,json)
S3method(ms_points,sf)
S3method(ms_points,sfc)
S3method(ms_simplify,SpatialLines)
S3method(ms_simplify,SpatialPolygons)
S3method(ms_simplify,character)
S3method(ms_simplify,json)
S3method(ms_simplify,sf)
S3method(ms_simplify,sfc)
export(apply_mapshaper_commands)
export(check_sys_mapshaper)
export(drop_null_geometries)
export(ms_clip)
export(ms_dissolve)
export(ms_erase)
export(ms_explode)
export(ms_filter_fields)
export(ms_filter_islands)
export(ms_innerlines)
export(ms_lines)
export(ms_points)
export(ms_simplify)
importClassesFrom(sp,SpatialLines)
importClassesFrom(sp,SpatialPoints)
importFrom(V8,v8)
importFrom(methods,.hasSlot)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,slot)
