#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom methods is .hasSlot as slot
#' @importFrom V8 v8
#' @importClassesFrom sp SpatialPoints SpatialLines
## usethis namespace: end
NULL
